/*-
 * Copyright (C) @BABOLO  2001 Jul 05
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * $Id: ra-filter.c,v 1.1 2001/07/05 03:38:41 babolo Exp $ 
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sysexits.h>
#include <sys/uio.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <netdb.h>
#include <stdio.h>
#include <err.h>


#define argtimeval timeval                       /* borrowed from argus (Carnegie Mellon University) */
#define arg_uint8   u_char
#define arg_int8    char
#define arg_uint16  u_short
#define arg_int16   short
#define arg_uint32  u_int
#define arg_int32   int                      /* END borrowed from argus (Carnegie Mellon University) */

#include <cons_def.h>
#include <cons_out.h>
#include "bufread.h"

typedef u_int32_t in_addr_t; /* For old FreeBSD */

char *myname;

int main(int argc, char *const *argv)
{   struct hostent *hp;
    struct in_addr addr;
    in_addr_t lookfor;
    int written, n;
    char *target;
    struct WriteStruct head, next;

    myname = argv[0];
    target = argv[1];
    written = 0;
    if  (inet_aton(target, &addr) != 0) lookfor = addr.s_addr;
    else
    {   hp = gethostbyname(target);
        if (!hp) errx(EX_NOHOST, "cannot resolve %s: %s", target, hstrerror(h_errno));
        lookfor =   ((hp->h_addr[3] & 0xFF) << 24)
                  | ((hp->h_addr[2] & 0xFF) << 16)
                  | ((hp->h_addr[1] & 0xFF) << 8)
                  | (hp->h_addr[0] & 0xFF);
    }
/*
    fprintf( stderr, "%u.%u.%u.%u\n"
           , (lookfor) & 0xFF
           , (lookfor >> 8) & 0xFF
           , (lookfor >> 16) & 0xFF
           , (lookfor >> 24) & 0xFF
           );
 */
    init_next();
    for (n = 0; get_next(fileno(stdin), &next) == sizeof(next); n++)
    {   if  (n == 0)
        {   if  (next.status & htonl(ARGUSCONTROL))
                bcopy(&next, &head, sizeof(next));
            else
            {   fprintf(stderr, "data error\n");                                      exit(EX_DATAERR);
            }
        }else if (next.status & htonl(IPPROTO))
        {   if  (  (next.ws_ip_src.s_addr == lookfor)
                || (next.ws_ip_dst.s_addr == lookfor)
                )
            {   if  (!written)
                {   if  (sizeof(head) != write(fileno(stdout), &head, sizeof(head)))
                                                                  err( EX_IOERR, "can't write stdout");
                    written = 1;
                }
                if  (sizeof(next) != write(fileno(stdout), &next, sizeof(next)))
                                                                  err( EX_IOERR, "can't write stdout");
    }   }   }
    close(fileno(stdout));
    exit(EX_OK);
}
